/*
** dist.c -- re-distribute a message
**
** This code is Copyright (c) 2002, by the authors of nmh.  See the
** COPYRIGHT file in the root directory of the nmh distribution for
** complete copyright information.
*/

#include <h/mh.h>
#include <h/utils.h>
#include <fcntl.h>

static struct swit switches[] = {
#define ANNOSW  0
	{ "annotate", 0 },
#define NANNOSW  1
	{ "noannotate", 0 },
#define EDITRSW  2
	{ "editor editor", 0 },
#define NEDITSW  3
	{ "noedit", 0 },
#define FORMSW  4
	{ "form formfile", 0 },
#define WHATSW  5
	{ "whatnowproc program", 0 },
#define NWHATSW  6
	{ "nowhatnowproc", 0 },
#define VERSIONSW  7
	{ "version", 0 },
#define HELPSW  8
	{ "help", 0 },
#define FILESW  9
	{ "file file", -4 },  /* interface from msh */
	{ NULL, 0 }
};


int
main(int argc, char **argv)
{
	int anot = 0, nedit = 0;
	int nwhat = 0, in, out;
	char *cp, *cwd, *maildir, *msgnam;
	char *ed = NULL, *file = NULL, *folder = NULL;
	char *form = NULL, *msg = NULL, buf[BUFSIZ], drft[BUFSIZ];
	char **argp, **arguments;
	struct msgs *mp = NULL;
	char *nfs;

#ifdef LOCALE
	setlocale(LC_ALL, "");
#endif
	invo_name = mhbasename(argv[0]);

	/* read user profile/context */
	context_read();

	arguments = getarguments(invo_name, argc, argv, 1);
	argp = arguments;

	while ((cp = *argp++)) {
		if (*cp == '-') {
			switch (smatch(++cp, switches)) {
			case AMBIGSW:
				ambigsw(cp, switches);
				done(1);
			case UNKWNSW:
				adios(NULL, "-%s unknown", cp);

			case HELPSW:
				snprintf(buf, sizeof(buf), "%s [+folder] [msg] [switches]", invo_name);
				print_help(buf, switches, 1);
				done(1);
			case VERSIONSW:
				print_version(invo_name);
				done(1);

			case ANNOSW:
				anot++;
				continue;
			case NANNOSW:
				anot = 0;
				continue;

			case EDITRSW:
				if (!(ed = *argp++) || *ed == '-')
					adios(NULL, "missing argument to %s",
							argp[-2]);
				nedit = 0;
				continue;
			case NEDITSW:
				nedit++;
				continue;

			case WHATSW:
				if (!(whatnowproc = *argp++) || *whatnowproc == '-')
					adios(NULL, "missing argument to %s",
							argp[-2]);
				nwhat = 0;
				continue;
			case NWHATSW:
				nwhat++;
				continue;

			case FILESW:
				if (file)
					adios(NULL, "only one file at a time!");
				if (!(cp = *argp++) || *cp == '-')
					adios(NULL, "missing argument to %s",
							argp[-2]);
				file = getcpy(expanddir(cp));
				continue;
			case FORMSW:
				if (!(form = *argp++) || *form == '-')
					adios(NULL, "missing argument to %s",
							argp[-2]);
				continue;
			}
		}
		if (*cp == '+' || *cp == '@') {
			if (folder)
				adios(NULL, "only one folder at a time!");
			else
				folder = getcpy(expandfol(cp));
		} else {
			if (msg)
				adios(NULL, "only one message at a time!");
			else
				msg = cp;
		}
	}

	cwd = getcpy(pwd());

	if (file && (msg || folder))
		adios(NULL, "can't mix files and folders/msgs");

	strncpy(drft, m_draft(seq_beyond), sizeof(drft));
	if ((out = creat(drft, m_gmprot())) == NOTOK)
		adios(drft, "unable to create");

	nfs = new_fs(form, distcomps);
	if (write(out, nfs, strlen(nfs)) != strlen(nfs)) {
		adios(drft, "error writing");
	}
	close(out);

	if (file) {
		/*
		** Dist a file
		*/
		anot = 0;  /* don't want to annotate a file */
	} else {
		/*
		** Dist a message
		*/
		if (!msg)
			msg = seq_cur;
		if (!folder)
			folder = getcurfol();
		maildir = toabsdir(folder);

		if (chdir(maildir) == NOTOK)
			adios(maildir, "unable to change directory to");

		/* read folder and create message structure */
		if (!(mp = folder_read(folder)))
			adios(NULL, "unable to read folder %s", folder);

		/* check for empty folder */
		if (mp->nummsg == 0)
			adios(NULL, "no messages in %s", folder);

		/* parse the message range/sequence/name and set SELECTED */
		if (!m_convert(mp, msg))
			done(1);
		seq_setprev(mp);  /* set the previous-sequence */

		if (mp->numsel > 1)
			adios(NULL, "only one message at a time!");
	}

	msgnam = file ? file : getcpy(m_name(mp->lowsel));
	if ((in = open(msgnam, O_RDONLY)) == NOTOK)
		adios(msgnam, "unable to open message");

	if (!file) {
		context_replace(curfolder, folder); /* update current folder */
		seq_setcur(mp, mp->lowsel);  /* update current message */
		seq_save(mp);  /* synchronize sequences  */
		context_save();  /* save the context file  */
	}

	if (nwhat)
		done(0);
	what_now(ed, nedit, NOUSE, drft, msgnam, 1, mp, anot ? "Resent" : NULL,
			cwd);
	done(1);
	return 1;
}
